@@ -85,6 +85,9 @@
             nb_setval/2,                        % +Var, +Value
             thread_create/2,                    % :Goal, -Id
             thread_join/1,                      % +Id
+            transaction/1,                      % :Goal
+            transaction/3,                      % :Goal, :Constraint, +Mutex
+            snapshot/1,                         % :Goal
             set_prolog_gc_thread/1,		% +Status
 
             '$wrap_predicate'/5                 % :Head, +Name, -Closure, -Wrapped, +Body
@@ -93,7 +96,10 @@
 :- meta_predicate
     dynamic(:, +),
     use_foreign_library(:),
-    use_foreign_library(:, +).
+    use_foreign_library(:, +),
+    transaction(0),
+    transaction(0,0,+),
+    snapshot(0).
 
 
                 /********************************
@@ -1516,6 +1522,19 @@ set_prolog_gc_thread(stop) :-
 set_prolog_gc_thread(Status) :-
     '$domain_error'(gc_thread, Status).
 
+%!  transaction(:Goal).
+%!  snapshot(:Goal).
+%
+%   Wrappers to guarantee clean Module:Goal terms.
+
+transaction(Goal) :-
+    '$transaction'(Goal).
+transaction(Goal, Constraint, Mutex) :-
+    '$transaction'(Goal, Constraint, Mutex).
+snapshot(Goal) :-
+    '$snapshot'(Goal).
+
+
 %!  '$wrap_predicate'(:Head, +Name, -Closure, -Wrapped, +Body) is det.
 %
 %   Would be nicer to have this   from library(prolog_wrap), but we need
