@@ -140,11 +140,11 @@ permission_error(Operation, PermissionType, Culprit) :-
 %   is, but if some variables are bound   to appropriate values it would
 %   be acceptable.
 %
-%   @param  Term is the term that needs (further) instantiation.
-%           Unfortunately, the ISO error does not allow for passing
-%           this term along with the error, but we pass it to this
-%           predicate for documentation purposes and to allow for
-%           future enhancement.
+%   @param  FormalSubTerm is the term that needs (further)
+%           instantiation. Unfortunately, the ISO error does not allow
+%           for passing this term along with the error, but we pass it
+%           to this predicate for documentation purposes and to allow
+%           for future enhancement.
 
 instantiation_error(_FormalSubTerm) :-
     throw(error(instantiation_error, _)).
@@ -229,6 +229,7 @@ resource_error(Resource) :-
 %   | negative_integer | Integer < 0 |
 %   | nonneg | Integer >= 0 |
 %   | oneof(L) | Ground term that is member of L |
+%   | pair | Key-Value pair |
 %   | positive_integer | Integer > 0 |
 %   | proper_list | Same as list |
 %   | list(Type) | Proper list with elements of Type |
@@ -374,6 +375,7 @@ has_type(negative_integer, X)     :- integer(X), X < 0.
 has_type(nonvar, X)       :- nonvar(X).
 has_type(number, X)       :- number(X).
 has_type(oneof(L), X)     :- ground(X), \+ \+ memberchk(X, L).
+has_type(pair, X)         :- nonvar(X), X = _-_.
 has_type(proper_list, X)  :- is_list(X).
 has_type(list, X)         :- is_list(X).
 has_type(list_or_partial_list, X)  :- is_list_or_partial_list(X).
