@@ -213,6 +213,9 @@ const debug_topic debug_topics[] =
 
   DEBUG_TOPIC(MSG_THREAD_WAIT),
 
+  DEBUG_TOPIC(MSG_TRANSACTION),
+  DEBUG_TOPIC(MSG_COMMIT),
+
   DEBUG_TOPIC(CHK_SECURE),
   DEBUG_TOPIC(CHK_HIGH_ARITY),
   DEBUG_TOPIC(CHK_HIGHER_ADDRESS),
