@@ -89,8 +89,7 @@ typedef struct
 		 *******************************/
 
 struct PL_global_data
-{ uintptr_t	heap_base;		/* heap-base rounded downwards */
-  pl_defaults_t	defaults;		/* system default settings */
+{ pl_defaults_t	defaults;		/* system default settings */
   pl_options_t	options;		/* command-line options */
   State		stateList;		/* list of loaded states */
   int		initialised;		/* Heap is initialised */
@@ -310,6 +309,7 @@ struct PL_global_data
     Procedure	is2;			/* is/2 */
     Procedure	strict_equal2;		/* ==/2 */
     Procedure	not_strict_equal2;	/* \==/2 */
+    Procedure   arg3;			/* arg/3 */
     Procedure	exception_hook4;
     Procedure	print_message2;
     Procedure	foreign_registered2;	/* $foreign_registered/2 */
